package compbio.stat.collector;

import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

import compbio.ws.client.Services;

public class StatManager {

	static class DateRoller implements Iterator<Date> {
		final Date initDate;
		final Calendar calendar;

		public DateRoller(Date date) {
			this.initDate = date;
			calendar = GregorianCalendar.getInstance();
			calendar.setTime(date);
			calendar.add(Calendar.MONTH, -12);
		}

		Date getCurrentDate() {
			return initDate;
		}

		@Override
		public boolean hasNext() {
			return !calendar.getTime().equals(initDate);
		}

		@Override
		public Date next() {
			calendar.add(Calendar.MONTH, 1);
			return calendar.getTime();
		}

		@Override
		public void remove() {
			throw new UnsupportedOperationException();
		}

	}

	void getStats() throws SQLException {
		Calendar startTime = Calendar.getInstance();
		startTime.roll(Calendar.YEAR, false);
		Timestamp startDate = new Timestamp(startTime.getTimeInMillis());
		Timestamp stopDate = new Timestamp(new Date().getTime());
		StatDB statdb = null;

		statdb = new StatDB();

		// Total
		Map<Services, StatProcessor> stats = new TreeMap<Services, StatProcessor>();
		for (Services service : Services.values()) {
			stats.put(
					service,
					new StatProcessor(statdb.readData(startDate, stopDate,
							service, null)));
		}

		// Cluster
		Map<Services, StatProcessor> statsCluster = new TreeMap<Services, StatProcessor>();
		for (Services service : Services.values()) {
			statsCluster.put(
					service,
					new StatProcessor(statdb.readData(startDate, stopDate,
							service, true)));
		}
		// Local
		Map<Services, StatProcessor> statsLocal = new TreeMap<Services, StatProcessor>();
		for (Services service : Services.values()) {
			statsLocal.put(
					service,
					new StatProcessor(statdb.readData(startDate, stopDate,
							service, false)));
		}

	}
}
